--[[
	Glyph Reporter
	
	Coded by Earl_Parvisjam

	This addon reports known and needed glyphs for all characters on the current server.
	As of version 1.6, this will also include glyphs for connected realms.
   ]]
   
GlyphReporter_Version = "1.6.1";
ReporterDB = {};
GR_Color = { ["name"] = "Red", ["r"] = 1, ["g"] = 0, ["b"] = 0 };
local realm = GetRealmName();
local player = UnitName("player");

---------------------------------------------------
-- OnLoad
---------------------------------------------------
function GlyphReporter_OnLoad(self)
	self:RegisterEvent("GLYPH_ENABLED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	StaticPopupDialogs["RESET_DATABASE"] = {
		text = "Are you sure you want to clear out all player info from the database?",
		button1 = "Yes",
		button2 = "No",
		showAlert = true;
		OnAccept = function()
				ReporterDB = {};
				ReporterDB[realm] = {};
				ReporterDB[realm][player] = {};
				GlyphReporter_GrabCurrentData();
			end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3,  
}
	
	GR_Print("Glyph Reporter v" ..GlyphReporter_Version .. " has been loaded successfully.");
end

---------------------------------------------------
-- OnEvent
---------------------------------------------------
function GlyphReporter_OnEvent(self, event)
	if (event == "PLAYER_ENTERING_WORLD") then
		if (ReporterDB[realm] == nil) then
			ReporterDB[realm] = {};
			ReporterDB[realm][player] = {};
		end
		GlyphReporter_GrabCurrentData();
	end
end

---------------------------------------------------
-- GrabCurrentData
-- Summary:  Grabs all current Glyph info
---------------------------------------------------
function GlyphReporter_GrabCurrentData()
	local glyphList = {};
	if (GetNumGlyphs() > 0) then
		for index = 1, GetNumGlyphs() do
			local name, glyphType, isKnown, _, id, glyphLink = GetGlyphInfo(index);
			if (name ~= "header") then
				glyphList[id] = { ["Name"] = name, ["Type"] = glyphType, ["Known"] = isKnown };
			end	
		end
	end
	
	local playerClass, englishClass = UnitClass("player");
	local Player = { ["Class"] = englishClass, ["Glyphs"] = glyphList };
	ReporterDB[realm][player] = Player;
end

---------------------------------------------------
-- CheckNeed
-- Summary:  Grabs all needy characters
---------------------------------------------------
local function GlyphReporter_CheckNeed(glyphId)
	GlyphReporter_GrabCurrentData();
	local needList = "Needed by:  \n";
	local spellId, class = CheckGlyphClass(glyphId);
	if (spellId ~= nil) then
		for currentPlayer, value in pairs(ReporterDB[GetRealmName()]) do
			if (value["Class"] == class) then
				if (value["Glyphs"][spellId] ~= nil) and not(value["Glyphs"][spellId].Known) then				
					needList = needList ..currentPlayer;
				end
			end
		end
		if (GetLocale() == 'enUS') then
			needList = needList .. ' ';
			local first = true;
			for i, currentRealm in ipairs(ConnectedRealmsNA[GetRealmName()]) do
				for currentPlayer, value in pairs(ReporterDB[currentRealm]) do
					if (value["Class"] == class) then
						if (value["Glyphs"][spellId] ~= nil) and not(value["Glyphs"][spellId].Known) then	
								if (first) then 
								needList = needList .. "\n";
							end
							needList = needList .. currentRealm .. ":" .. currentPlayer .. ' ';
						end
					end
				end
				first = false;
			end
		end
	else
		GR_Print("Glyph ID missing from database:  " .. glyphId);
	end
	return needList;
end

---------------------------------------------------
-- GR_Print
-- Summary:  A simple print wrapper mostly used in debugging
---------------------------------------------------
function GR_Print(msg)
	if (msg == nil) then return; end
	if( DEFAULT_CHAT_FRAME ) then DEFAULT_CHAT_FRAME:AddMessage(msg, 1,1, 0); end
end

setOnce = {};
---------------------------------------------------
-- ToolTipHooker
-- Summary:  Hooks extra text onto the tooltip
---------------------------------------------------
local function ToolTipHooker(self, ...)
	local _, _, _, _, _, ItemType = GetItemInfo(43364);
	if (not setOnce[self]) then
		local itemLink = select(2, self:GetItem());
		if itemLink and GetItemInfo(itemLink) then	
			local itemInfo = string.match(itemLink, "item[%-?%d:]+");
			local first, itemId, third  = strsplit(":", itemInfo, 3);
			local itemName, _, _, _, _, itemType = GetItemInfo(itemLink);
			if (ItemType == itemType) then
				local list = GlyphReporter_CheckNeed(itemId);
				self:AddLine(list, GR_Color.r, GR_Color.g, GR_Color.b);
				setOnce[self] = true;
			end

		end
	end
end

---------------------------------------------------
-- ToolTipUnHooker
-- Summary:  Unhooks extra text on the tooltip
---------------------------------------------------
local function ToolTipUnhooker(self, ...)
	setOnce[self] = false;
end

GameTooltip:HookScript("OnTooltipSetItem", ToolTipHooker);
GameTooltip:HookScript("OnTooltipCleared", ToolTipUnhooker);

---------------------------------------------------
-- ***Interface Options Framework***
---------------------------------------------------

local colorList = { 
	["White"] = { ["r"] = 1, ["g"] = 1, ["b"] = 1,},
	["Yellow"] = { ["r"] = 1, ["g"] = 1, ["b"] = 0,},
	["Red"] = { ["r"] = 1, ["g"] = 0, ["b"] = 0,},
	["Green"] = { ["r"] = 0, ["g"] = 1, ["b"] = 0,},
	["Blue"] = { ["r"] = 0, ["g"] = 0, ["b"] = 1,},
}

---------------------------------------------------
-- GR_GUIFrame_OnLoad
---------------------------------------------------
function GR_GUIFrame_OnLoad(panel)
	panel.name = "Glyph Reporter";
	panel.okay = function (self) GR_ColorSelectButton_OnClick(); end;
	panel.cancel = function (self) GR_ColorSelectButton_Close(); end
	InterfaceOptions_AddCategory(panel);
end

---------------------------------------------------
-- GR_ColorSelect_OnClick
---------------------------------------------------

local function GR_ColorSelectButton_OnClick(self)
	--UIDropDownMenu_SetText(self, self.value);
	ToggleDropDownMenu(1, nil, GR_ColorSelectButton, "GR_ColorSelectButton", 0, 0);
end

---------------------------------------------------
-- GR_ColorClick
-- Summary:  Action taken when the button is clicked
---------------------------------------------------
function GR_ColorClick(self)
	GR_Color.name = self.value;
	self.checked = true;
	GR_Color.r = colorList[self.value].r;
	GR_Color.g = colorList[self.value].g;
	GR_Color.b = colorList[self.value].b;
	ToggleDropDownMenu(1, nil, GR_ColorSelectButton, "GR_ColorSelectButton", 0, 1);
end

function GR_ColorSelectButton_Close(self)
	UIDropDownMenu_SetText(self, self.value);
end

---------------------------------------------------
-- GR_ColorSwatchButton_OnClick
-- Summary:  A swatch that allows selection of color values
---------------------------------------------------
function GR_ColorSwatchButton_OnClick(self)
	ColorPickerFrame.previousValues = {GR_Color.r, GR_Color.g, GR_Color.b, 1};
	ColorPickerFrame:SetColorRGB(GR_Color.r, GR_Color.g, GR_Color.b);
	ColorPickerFrame.hasOpacity = false;
	ColorPickerFrame:Hide();
	ColorPickerFrame:Show();
	ColorPickerFrame.func = UpdateColors;
	ColorPickerFrame.cancelFunc = UpdateColors;
end

function UpdateColors(reset)
	if (reset) then
		GR_Color.r, GR_Color.g, GR_Color.b = unpack(reset);
	else
		GR_Color.r, GR_Color.g, GR_Color.b = ColorPickerFrame:GetColorRGB();
	end	
end

---------------------------------------------------
-- CheckGlyphClass
-- Summary:  Determines the current Glyph's Class 
---------------------------------------------------
function CheckGlyphClass(glyphId)
	local classValue = "undetermined";
	local spellId = "unspecified";
	if (DKData[glyphId] ~= nil) then
		classValue = "DEATHKNIGHT";
		spellId = DKData[glyphId];
	elseif (DruidData[glyphId] ~= nil) then
		classValue = "DRUID";
		spellId = DruidData[glyphId];
	elseif (HunterData[glyphId] ~= nil) then
		classValue = "HUNTER";
		spellId = HunterData[glyphId];
	elseif (MageData[glyphId] ~= nil) then
		classValue = "MAGE";
		spellId = MageData[glyphId];
	elseif (MonkData[glyphId] ~= nil) then
		classValue = "MONK";
		spellId = MonkData[glyphId];
	elseif (PaladinData[glyphId] ~= nil) then
		classValue = "PALADIN";	
		spellId = PaladinData[glyphId];
	elseif (PriestData[glyphId] ~= nil) then
		classValue = "PRIEST";	
		spellId = PriestData[glyphId];
	elseif (RogueData[glyphId] ~= nil) then
		classValue = "ROGUE";	
		spellId = RogueData[glyphId];
	elseif (ShamanData[glyphId] ~= nil) then
		classValue = "SHAMAN";
		spellId = ShamanData[glyphId];
	elseif (WarlockData[glyphId] ~= nil) then
		classValue = "WARLOCK";
		spellId = WarlockData[glyphId];
	elseif (WarriorData[glyphId] ~= nil) then
		classValue = "WARRIOR";
		spellId = WarriorData[glyphId];
	else	
		classValue = "ERROR";
		spellId = "UNSPECIFIED";
	end
	return spellId, classValue;
end

